;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xc7d0ef0652e8adabd59890c462007b4a . "./code.cg2js/cg-status-bar.fasl")
(#xf1e4610665889fc23ebaad7e507a7e37 . "./code.cg2js/cg-hotspot.fasl")
(#x1442bba87ee5ccb9487335a188a6db80 . "./code.cg2js/cg-toolbar.fasl")
(#xb16e701532119fbc23611ca897d1d98f . "./code.cg2js/cg-clipboard-stack.fasl")
(#x41ddba9d000904af1105307320eeab95 . "./code.cg2js/cg-tooltip.fasl")
(#xfa5f4551f0c6e7eae1e99e4614865635 . "./code.cg2js/cg-calendar.fasl")
(#x17caf3116ff09d6d3437229c516ff5c9 . "./code.cg2js/cg-item-list.fasl")
(#x3f69a6fa7cdace8dfd9e242e4864df34 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xfccfe81895d4fa956d0264cf57bf76a2 . "./code.cg2js/cg-trackbar.fasl")
(#x7d68f0b41b0a81df4322f4dce55bbe30 . "./code.cg2js/cg-class-grid.fasl")
(#x74a4dfcdbc60dda7162cd2d23740ebe1 . "./code.cg2js/cg-dialog-item.fasl")
(#xabe8c43097904c13c4b4648199758f72 . "./code.cg2js/cg-comtab.fasl")
(#x6b8def59162e0900763039187a09fd8c . "./code.cg2js/cg-lisp-text.fasl")
(#x0d40d99855c79d68518f693152747b5b . "./code.cg2js/cg-button.fasl")
(#xf856e7c635def44e78d88207bd859030 . "./code.cg2js/cg-printing.fasl")
(#xab387a35c21626f639d391c540e4fa18 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x6656289a98e8d5a6196992af82dd058e . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x1e7c97fa7ad795d5c4be1c8d19673123 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xa4206dcbfe3bc744d8c3a382c2b1cea7 . "./code.cg2js/cg-selected-object.fasl")
(#xe5bec6c029e807c43911c124c3274e9d . "./code.cg2js/cg-text-or-combo.fasl")
(#x37f31790e7d09a98c8d1efb98ed9a8c2 . "./code.cg2js/cg-graphics-context.fasl")
(#x56bec55e53abbca129db14168f08435d . "./code.cg2js/cg-edit-in-place.fasl")
(#x4f924345c555952f1f732a0ae9a20190 . "./code.cg2js/cg-string-dialog.fasl")
(#x652529e6b426b079b38627995fbad4fa . "./code.cg2js/cg-list-view.fasl")
(#xa4666335d0365c890a4ab9c753a60153 . "./code.cg2js/cg-intersect.fasl")
(#x898b23a0e8817ba8535f3a855587193f . "./code.cg2js/cg-text-widget.fasl")
(#x2d093c5c9f6afa858d7ab0a57afb582d . "./code.cg2js/cg-directory-dialog.fasl")
(#xc2e409c2db76d60e1b65034beb2f714e . "./code.cg2js/cg-file-dialog.fasl")
(#xc2454c1b983f78cddc8d73f944e9e583 . "./code.cg2js/cg-dde-utils.fasl")
(#x6e04d36d4463c79ec6a1ea8ed099b19f . "./code.cg2js/cg-nodes.fasl")
(#x72693cff6092a33c99d35468e54d8210 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xade9a3b3c0da8d88863084eb1199b30e . "./code.cg2js/cg-up-down-control.fasl")
(#x716ca0f9100e442cf3eae2326d2fe1b0 . "./code.cg2js/cg-get-position.fasl")
(#x4ee8293c411e91ecd76541b36bd9be21 . "./code.cg2js/cg-toggling-widget.fasl")
(#xc49752c5aaae71452c7b4864ada5da31 . "./code.cg2js/cg-acache.fasl")
(#x63a95494ce7c509a970d0c38f52418eb . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x9e37ddbe0fed3d870542e42c1d49c74d . "./code.cg2js/cg-base.fasl")
(#xe97d323a4dbdd80cc7434673e959ea2a . "./code.cg2js/cg-drag-and-drop.fasl")
(#xda44be277bcb1a0c6a22c4b1eb6adc81 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x2f59e09d78dd3a0dea798a2588c031fb . "./code.cg2js/cg-property.fasl")
(#x4089138b5a9a2fcb149e3f3ae6eb001d . "./code.cg2js/cg-editable-text.fasl")
(#x6fb15ef6efec7da8029901be9237838e . "./code.cg2js/cg-lamp.fasl")
(#x4aaaa093111fb2d66a2db4fb5c78f5ef . "./code.cg2js/cg-progress-indicator.fasl")
(#x308e3b584af50973e860b2340790bae4 . "./code.cg2js/cg-chart-widget.fasl")
(#x9723f43c57d38cf5cf767d819908b58d . "./code.cg2js/cg-common-control.fasl")
(#xe669cef297f32fcad6ad0be516c9ce6c . "./code.cg2js/cg-font-dialog.fasl")
(#x9d509c37931d0b211fdf391dbd291366 . "./code.cg2js/cg-gesture-emulation.fasl")
(#xf69d4dc26637791f12564a0c66daa267 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x47e30acb2ada307edb70d1f383da5bc6 . "./code.cg2js/cg-os-window.fasl")
(#x6d74237f6c0dfa49cf68472bd746171b . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x013ead60336acb7a514c144fd0c829f6 . "./code.cg2js/cg-pixmap.fasl")
(#x9bf94b9c15b990bd5ce17e9a7692aeb5 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x64e16b4e185d4cf6b6986700e633d337 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xdb95fb16c15f0414d12a2c45d157de5e . "./code.cg2js/cg-scaling-stream.fasl")
(#xc248ba350158c26437058e0e42a95fdb . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x19e651610410350ac769991c19b0108d . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x1a2afef15c4ec0f96e23b9208df71fe1 . "./code.cg2js/cg-timer.fasl")
(#x8468aec95abef51a7ef6021926ad09ce . "./code.cg2js/cg-scroll-bar.fasl")
(#x5e19ba5929b7139c8d0b6f3ce5d5dfdd . "./code.cg2js/cg-caret.fasl")
(#x0080284743a7f4b6348e6f4781184fdc . "./code.cg2js/cg-template-string.fasl")
(#xbd843c336856cdb095399469e08c4c17 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x844113a07e369f84a21063e0667e9096 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x6b0a949d97bc1fcb5e25bd6ca30fc368 . "./code.cg2js/cg-combo-box.fasl")
(#xca3107f87a3361e3a4f8a74b7ee38054 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x4cd978e60ec3702257cb772e28675fbd . "./code.cg2js/cg-multi-picture-button.fasl")
(#x93773275996e3843c2d42de203ba899c . "./code.cg2js/cg-menu-tooltip.fasl")
(#x0e0b632947a390cf71ee6e9dcd9f7580 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x6e4f414ac05977b454cd390813f3ce8b . "./code.cg2js/cg-player.fasl")
(#x5d3a559736a422b146388d8f26324a7c . "./code.cg2js/cg-outline.fasl")
(#xc0b3492628df6da1d1595efb9c9e6602 . "./code.cg2js/cg-class-support.fasl")
(#xf8a2fa8a0623f7c1d84a8c65a3d5b2d1 . "./code.cg2js/cg-lettered-menu.fasl")
(#xb85b541f6e309bc914f1b27119a3bb0c . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x149b100adad6dfaab03b07ea0b896f4e . "./code.cg2js/cg-fill-texture.fasl")
(#xebac33d1ca6ebe1678fc9d8355c53475 . "./code.cg2js/cg-lisp-widget.fasl")
(#xd37ccfa7079e0e44b54320da4387e861 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x16b410d87a79518b6eb53b63c090dbb3 . "./code.cg2js/cg-clipboard.fasl")
(#x167d640a5ca09df35558868a59f6f726 . "./code.cg2js/cg-wrap-string.fasl")
(#x6ada770a9bd37857042b8c4fa1d8061d . "./code.cg2js/cg-os-widget.fasl")
(#xe690660b75826f00b918d643d4da6c78 . "./code.cg2js/cg-check-box.fasl")
(#x9ddc66dfa4b5ec638e389349cf36362a . "./code.cg2js/cg-choice-list.fasl")
(#x0d7b3cde217147a4488bd594dc883b02 . "./code.cg2js/cg-layout.fasl")
(#xa971ff9c92fad02b395d8dbddb643927 . "./code.cg2js/cg-group-box.fasl")
(#xd37b6db3a52ff7cf2001a256e161fc18 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x1e477b19dd235dc1aa500b2436f6ba3a . "./code.cg2js/cg-radio-button.fasl")
(#x119e51c95b70fcb56fb5806d8a2e9613 . "./code.cg2js/cg-tab-control.fasl")
(#x5d3594f9ec6fee6fa1853f4b7d0cc538 . "./code.cg2js/cg.fasl")
(#x7f22cef5a6c100972b9aa22c4525e000 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x20f1b0940b7a32d5d511debf164af3be . "./code.cg2js/cg-dropping-outline.fasl")
(#xdb6670627f7ebd7597d38b36ff518f7a . "./code.cg2js/cg-get-pixmap.fasl")
(#xfea341e261a2267979839148e9a4ce77 . "./code.cg2js/cg-grid-widget.fasl")
(#x450721cf0b52ddc4c17388797857d0ff . "./code.cg2js/cg-utility-dialog.fasl")
(#x17e3f0a93e5c3a80fa97a5c10e3eba82 . "./code.cg2js/cg-static-text.fasl")
(#xcac54265e2566204a328e82ae2916c23 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xecf9a3367045221147b557794428c0fa . "./code.cg2js/cg-object-editor-layout.fasl")
(#x8f6c208220be954fdf5bb8d82a7b3f6b . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xd3baddfe011c352ef1b55ed66b75b9da . "./code.cg2js/cg-bitmap-pane.fasl")
(#x35d92e71d84a769de872c4d4ac4fd350 . "./code.cg2js/cg-plot-widget.fasl")
(#x399d2c3ef04c6c9a66ef603c2fa4cf7a . "./code.cg2js/cg-color-dialog.fasl")
(#xdc6c767e8f1a771e5d57f5fb6023bf26 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xd173be79a9f4d762747c11d7d4e0ae72 . "./code.cg2js/cg-icon.fasl")
(#x1f7bc565d0789c6c30dbe88e6ec57162 . "./code.cg2js/cg-dev.fasl")
(#x559f93314150a8a776fafeb1da052771 . "./code.cg2js/cg-choose-printer.fasl")
(#x36b4d1ca1a7bd0e1e0d745cfaf10692a . "./code.cg2js/cg-intersect-posbox.fasl")
(#x10c41710c938210597da80246b869762 . "./code.cg2js/cg-menu.fasl")
(#x60e4cd055b311c3091e93a08fd452904 . "./code.cg2js/cg-yes-no-list.fasl")
(#x453536b38aeef5f769599c2c0c0e9a5e . "./code.cg2js/cg-palette.fasl")
(#x355cb1d6cd91c3f63818959348bd8dde . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x9d18f5bc09620bd6f574f8a34abf7fbd . "./code.cg2js/cg-message-dialog.fasl")
(#xe4f7e645f24fb7768423c9dd97e2fe48 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xc4be64c0d22e064f506b9f7c03c42c4a . "./code.cg2js/cg-split-bar.fasl")
(#xff146931f6b8a0ffdbb903477a30ec0e . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xf8c6467eface542353aec2560c63c255 . "./code.cg2js/cg-web-browser.fasl")
(#x2afa314e601ca038aacaee313aac1b63 . "./code.cg2js/cg-project-window.fasl")
(#x44de786ea3cf267ee7796c41fd6b5fe7 . "./code.cg2js/cg-picture-widget.fasl")
(#x14105dc5f03911a1acd7ab1ed163cc5b . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x965cf2248a02f5621bdae25e72213886 . "./code.cg2js/cg-object-editor.fasl")
(#x3bf63d1efee16ecf62c620f843d62c77 . "./code.cg2js/cg-paren-matching.fasl")
(#xf9da5157a5728bd33e473c51c9b3f8ca . "./code.cg2js/cg-sample-file-menu.fasl")
(#x34a1b17166ca13511a8d44a598850480 . "./code.cg2js/cg-drawable.fasl")
(#xb186e61b7903b956d9817bb11cfb0e3e . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x221ee21a10c04c905895dae9151248c8 . "./code.cg2js/cg-html-widget.fasl")
(#xcb10dfb007189c4cf11c4c483d6c609a . "./code.cg2js/cg-curve.fasl")
(#x38d63c8bbdc51fe676b98ac32517be24 . "./code.cg2js/cg-yes-no-string.fasl")
(#x7c5dd565337da63588cecd2164ec76e5 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xcc5cc3553f0439d0c0061046fb0b6a27 . "./code.cg2js/cg-html-dialog.fasl")
(#xa717f30f4302256059c840f198f52c52 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x064ea51ba5690312efcc13553b7e2115 . "./code.cg2js/cg-tray.fasl")
(#x9d5e22d64d5b32f7483c754e868a9c63 . "./code.idejs/ide-apropos.fasl")
(#xb3ae65da3db03bdc9edf3fa29c2ccb8e . "./code.idejs/ide-window-list.fasl")
(#x9c7a3dc401862956b6e40cb2a211bb8f . "./code.idejs/ide-class-browser.fasl")
(#x7bbd70ed884109e36ab0dbf7d6ebea14 . "./code.idejs/ide-base.fasl")
(#x98b4aed7a94ef59f4cbef6cdbfeca315 . "./code.idejs/ide-process-browser.fasl")
(#xd4ccce17e566a22ad8eca5c342c310b9 . "./code.idejs/ide-acache.fasl")
(#x3d0361ee2fb6b00a0cbd71f36e7dcb94 . "./code.idejs/ide-navigator.fasl")
(#x07995b4d68014d209ab31744840c0460 . "./code.idejs/ide-trace.fasl")
(#x330cd1571ee440282a4316c4be8a99aa . "./code.idejs/ide-project.fasl")
(#xf9ddfb89ee9c65600fd44ff82ed54c5a . "./code.idejs/ide-patch-dialog.fasl")
(#xa20432152e9e7f67482aff4350dd2fdd . "./code.idejs/ide-grapher.fasl")
(#xc28d803fd1090b07fb679c400df956af . "./code.idejs/ide-dev.fasl")
(#x54a25fc0f5495c71713924e0e1cd6d30 . "./code.idejs/ide-keyshort.fasl")
(#x2a8f7756f911acb422ae58e8c1e1de00 . "./code.idejs/ide-editor.fasl")
(#x9d9700d5f2520f7cb255410321a6aa9f . "./code.idejs/ide-inspector.fasl")
(#x92279ac2d6524cf8ea9b229f8c0b9d68 . "./code.idejs/ide.fasl")
(#xae679b8fd1ed23518922c17dc5381acf . "./code.idejs/ide-clipboard.fasl")
(#x5c70bbdcf126108732cd33e83de1b87c . "./code.idejs/ide-find-definitions.fasl")
(#x1990db4faceddd4ec38f45a547869860 . "./code.idejs/ide-form.fasl")
(#x5caa716bdffa5859183ade14072244d4 . "./code.idejs/ide-intro.fasl")
(#xa71e9c0c8feef04a87a844173ae04542 . "./code.idejs/ide-find-in-files.fasl")
(#x9e01b6907f92926333c533a63b927d87 . "./code.idejs/ide-package-list.fasl")
(#x33edc8374b6a23bc802adabfd7363930 . "./code.idejs/ide-class-grapher.fasl")
(#x9a8f50ffdba20a10156ef75920aa924b . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (format t "Incorrect checksum on file: ~a" (cdr checksum))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
